#========================================================================================
# package      DE_EPAGES::Core::API::GitHub
# state        public
#----------------------------------------------------------------------------------------
# description  This module provides GitHub API functionality.
#----------------------------------------------------------------------------------------
# example      my $GitHub = DE_EPAGES::Core::API::GitHub->new($User, $Password);
#========================================================================================
package DE_EPAGES::Core::API::GitHub;
use base qw( Exporter );

use strict;
use DE_EPAGES::WebService::API::JSONRestClient; ## no critic qw(RequireCorrectImports)

our @EXPORT_OK = qw (
    CreatePullRequest
    ApprovePullRequest
    MergePullRequest
    IsMergeable
    IsMerged
);

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $GitHub = DE_EPAGES::Core::API::GitHub->new();
#----------------------------------------------------------------------------------------
# description  Create a GitHub object.
#----------------------------------------------------------------------------------------
# input        $User | username of the GitHub account to push changes to | string
# input        $Password | password or API token of the GitHub account | string
# input        $Account | optional; the owner GitHub account of the target repository.
#                   Default: $User | string
# return       $GitHubObject | GitHub object | object
#========================================================================================
sub new {
    my $class = shift;
    my ($User, $Password, $Account) = @_;

    $Account //= $User;

    my $self->{'BaseUrl'} = 'https://api.github.com';
    $self->{'User'} = $User;
    $self->{'Account'} = $Account;
    $self->{'BasicAuthentication'} = [$User, $Password];

    return bless $self, $class;
}

#========================================================================================
# function     CreatePullRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->CreatePullRequest($Title, $FeatureBranch, $Branch, $Comment);
#----------------------------------------------------------------------------------------
# description  Create a pull request of a given feature branch to branch.
#----------------------------------------------------------------------------------------
# input        $Title | title for the pull request | string
# input        $FeatureBranch | branch to create pull request from | string
#               e.g. 'build/version_6_17_33'
#               e.g. for other repositories/forks <github-username>:build/version_6_17_33
# input        $TargetBranch | the branch where the changes should be | string
# input        $Repo | repository where the branches are | string
# input        $Comment | any comments for the pull request | string
# return       $PRID | number of the pull request on GitHub | int
#========================================================================================
sub CreatePullRequest {
    my $self = shift;
    my ($Title, $FeatureBranch, $TargetBranch, $Repo, $Comment) = @_;

    #get post url
    my $PullUrl = sprintf('%s/repos/%s/%s/pulls',
                          $self->{'BaseUrl'}, $self->{'Account'}, $Repo);

    #build JSON parameter
    my $JSONData = {
        'title' => $Title,
        'head'  => $FeatureBranch,
        'base'  => $TargetBranch,
        'body'  => $Comment
    };

    my $JSONRestClient = $self->_createJSONRestClientWithBaseUrl($PullUrl);
    my $Response = $JSONRestClient->post($JSONData);

    return $Response->{'number'};
}

#========================================================================================
# function     ApprovePullRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->ApprovePullRequest($PRID, $Repo);
#----------------------------------------------------------------------------------------
# description  Creates a review for a pull request in GitHub.
#----------------------------------------------------------------------------------------
# input        $PRID | number of the pull request on GitHub | int
# input        $Repo | repository where the branches are | string
# return       $ApproveID | review ID of the pull request | string
#========================================================================================
sub ApprovePullRequest {
    my $self = shift;
    my ($PRID, $Repo) = @_;

    my $ApproveUrl = sprintf('%s/repos/%s/%s/pulls/%s/reviews',
                            $self->{'BaseUrl'}, $self->{'Account'}, $Repo, $PRID);
    my $JSONData = { 'event' => 'APPROVE' };

    my $JSONRestClient = $self->_createJSONRestClientWithBaseUrl($ApproveUrl);
    return $JSONRestClient->post($JSONData);
}

#========================================================================================
# function     MergePullRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->MergePullRequest($CommitMessage, $PRID, $Repo);
#----------------------------------------------------------------------------------------
# description  Merge a pull request in GitHub.
#----------------------------------------------------------------------------------------
# input        $CommitMessage | message to comment the merge commit | string
# input        $PRID | number of the pull request on GitHub | int
# input        $Repo | repository where the branches are | string
# return       $MergeID | commit ID of the merge commit | string
#========================================================================================
sub MergePullRequest {
    my $self = shift;
    my ($CommitMessage, $PRID, $Repo) = @_;

    my $MergeUrl = sprintf('repos/%s/%s/pulls/%s/merge', $self->{'Account'}, $Repo, $PRID);
    my $JSONData = {
        'commit_message' => $CommitMessage,
        'merge_method' => 'squash'
    };

    my $JSONRestClient = $self->_createJSONRestClient();
    my $Response = $JSONRestClient->put($MergeUrl, $JSONData, 1);

    return $Response->{'sha'};
}

#========================================================================================
# function     IsMergeable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->IsMergeable($PRID);
#----------------------------------------------------------------------------------------
# description  Check if a pull request is mergeable.
#----------------------------------------------------------------------------------------
# input        $PRID | number of the pull request on GitHub | int
# input        $Repo | repository where the branches are | string
# return       1, if pull request is mergeable, 0 otherwise | bool
#========================================================================================
sub IsMergeable {
    my $self = shift;
    my ($PRID, $Repo) = @_;

    my $JSONRestClient = $self->_createJSONRestClient();
    my $Response = $JSONRestClient->get(
        sprintf('repos/%s/%s/pulls/%s', $self->{'Account'}, $Repo, $PRID));

    return ( $Response->{'mergeable'} and not $Response->{'merged'} ) ? 1 : 0;
}

#========================================================================================
# function     IsMerged
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->IsMerged($PRID);
#----------------------------------------------------------------------------------------
# description  Check if a pull request is mergeable.
#----------------------------------------------------------------------------------------
# input        $PRID | number of the pull request on GitHub | int
# input        $Repo | repository where the branches are | string
# return       1, if pull request is mergeable, 0 otherwise | bool
#========================================================================================
sub IsMerged {
    my $self = shift;
    my ($PRID, $Repo) = @_;

    my $JSONRestClient = $self->_createJSONRestClient();
    my $Response = $JSONRestClient->get(
        sprintf('repos/%s/%s/pulls/%s', $self->{'Account'}, $Repo, $PRID));

    return $Response->{'merged'} ? 1 : 0;
}

sub _createJSONRestClient {
    my $self = shift;

    return DE_EPAGES::WebService::API::JSONRestClient->new($self->{'BaseUrl'},
        {'BasicAuthentication' => $self->{'BasicAuthentication'}});
}

sub _createJSONRestClientWithBaseUrl {
    my $self = shift;

    my ($BaseUrl) = @_;

    return DE_EPAGES::WebService::API::JSONRestClient->new($BaseUrl,
        {'BasicAuthentication' => $self->{'BasicAuthentication'}});
}

1;
