use strict;
use Getopt::Long;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::GitHub;

# A test for this script is defined in the infra repo:
# /epages-infra/scripts/epages/test/build/test_pull_request_creation.sh

sub Main {

    my $self = shift;
    local $| = 1;

    my $SourceBranch;
    my $TargetBranch;
    my $Account;
    my $UserName;
    my $UserToken;
    my $ApproverName;
    my $ApproverToken;
    my $PropertiesFile;
    my $Help;
    my $UserCrossRepo;
    my $mergePR = 0;
    my $PullRequestId;
    my $MergeId;


    GetOptions(
        'sourceBranch=s'        => \$SourceBranch,
        'targetBranch=s'        => \$TargetBranch,
        'account=s'             => \$Account,
        'userName=s'            => \$UserName,
        'userToken=s'           => \$UserToken,
        'approverName=s'        => \$ApproverName,
        'approverToken=s'       => \$ApproverToken,
        'userCrossRepo=s'       => \$UserCrossRepo,
        'propertiesFile=s'      => \$PropertiesFile,
        'help'                  => \$Help,
        'mergePR'               => \$mergePR,
    );

    usage() if $Help;

    unless( $SourceBranch ) {
        print "missing parameter -sourceBranch\n";
        usage();
    }

    unless( $TargetBranch ) {
        print "missing parameter -targetBranch\n";
        usage();
    }

    unless( $Account ) {
        $Account = $UserName;
    }

    unless( $UserName ) {
        print "missing parameter -userName\n";
        usage();
    }

    unless( $UserToken ) {
        print "missing parameter -userToken\n";
        usage();
    }

    if ( $ApproverName and not $ApproverToken ) {
        print "missing parameter -approverToken\n";
        usage();
    }

    $SourceBranch = $UserCrossRepo.':' . $SourceBranch if(defined $UserCrossRepo);
    my $Title = "Merge generated Patch Files from $SourceBranch";
    my $Repository = 'Cartridges';

    print "Create pull request to merge commits from branch '$SourceBranch' into '$TargetBranch' on account '$Account'.\n";

    my $GitHub = DE_EPAGES::Core::API::GitHub->new($UserName, $UserToken, $Account);

    # Create pull request
    $PullRequestId = $GitHub->CreatePullRequest($Title, $SourceBranch, $TargetBranch, $Repository);
    die 'Could not create pull request.' unless $PullRequestId;

    # Approve pull request
    if ($ApproverName) {
        my $GitHub2 = DE_EPAGES::Core::API::GitHub->new($ApproverName, $ApproverToken, $Account);
        $GitHub2->ApprovePullRequest($PullRequestId, $Repository);
    }

    # Merge pull request
    if ($mergePR) {
        for (1 .. 6) {
            if($GitHub->IsMergeable($PullRequestId, $Repository)) {
                my $CommitMessage = 'Merge generated patch files';
                $MergeId = $GitHub->MergePullRequest($CommitMessage, $PullRequestId, $Repository);
                last;
            }
            sleep(5);
        }
        if($GitHub->IsMerged($PullRequestId, $Repository)) {
            print "Pull request '$PullRequestId' successfully merged on GitHub.\n";
        } else {
            die "Pull request '$PullRequestId' on GitHub failed.\n";
        }
    }
}

sub usage {
    print <<END_USAGE;
This script creates a pull request from the given source branch to the given
target branch. It will also approve the pull request if -approverName is set.
It will also merge the pull request if -mergePR is set.

Usage:
    perl $0 [options] [flags]

Options:
    -sourceBranch     branch where your changes are implemented [req.]
    -targetBranch     branch you want your changes pulled into [req.]
                      (branch must exist in the <account> repository)
    -account          name of the GitHub accout where <sourceBranch> and
                      <targetBranch> are located [opt., default: <userName>]
    -userName         GitHub account of the user who creates the PR [req.]
    -userToken        personal access token for <userName> [req.]
    -approverName     GitHub account of the user who approves the PR [opt.]
                      (must be other account than <userName>)
    -approverToken    personal access token for <approverName> [opt.]
                      (must exist if <approverName> is set)
    -userCrossRepo    name of a different GitHub account than <userName> where
                      the <sourceBranch> is located [opt.]
    -propertiesFile   the path and name of the properties file [opt.]

Flags:
    -help             show the command line options
    -mergePR          merge the pull request directly

Example:
    perl $0 \
    -sourceBranch patch/7.52.0 -targetBranch develop -account ePages-de \
    -userName epages-****-bot -userToken ec94****ae3e \
    -approverName epages-****-bot-approver -approverToken 12a9****f1c5 -mergePR \
    -propertiesFile ~jenkins/workspace/build/buildPackages/cdp.properties

END_USAGE
    exit 2;
}

RunScript(
    'Debug' => 0,
    'Message' => 'short',
    'Sub' => \&Main
);

1;
