#==============================================================================
# package      DE_EPAGES::Product::UI::ProductType
# state        private
#==============================================================================
package DE_EPAGES::Product::UI::ProductType;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( IndexOf );
use DE_EPAGES::Core::API::Unicode qw (Unicode2Alias);
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias Transaction );
use DE_EPAGES::Object::API::Table::SelectPreDefAttribute qw ( GetPKeySelectPreDefAttributesByAttributeID );
use DE_EPAGES::Presentation::API::PageType qw (LoadPageTypeByAlias);
use DE_EPAGES::Product::API::Table::SelectedVariation qw ( GetPKeySelectedVariationsByAttributeID );
use DE_EPAGES::Product::UI::ProductAttribute;

sub ErrorDefinition {
    return {
        WrongAttributeClass => 'Invalid attribute class id: #AttributeClassID. Expected: #ProductTypeID',
    };
}

#==============================================================================
# Event handlers
#==============================================================================

sub NewAttribute {
    my $self = shift;
    my ($Servlet) = @_;
    my $ProductType = $Servlet->object;
    my $Parent = $ProductType->folder('Attributes');
    my $Class = LoadClassByAlias('ProductAttribute');

    eval {
        Transaction( sub {
            DE_EPAGES::Product::UI::ProductAttribute->NewObject( $Servlet, $Parent, $Class, {
                AttributeClass => $ProductType,
            } );
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code =~ /VariationFeatureLimitExceeded/){
            $Servlet->vars('VariationFeatureMaxValueReached',1);
            Warning($Error);
        } elsif ($Error->code  eq 'FeatureLimitExceeded'){
            $Servlet->vars('FeatureMaxValueReached', $Error->vars);
            Warning($Error);

        } else {
            Error($Error);
        }
    }

    return;
}

sub UnityNewAttribute {
    my $self = shift;
    my ($Servlet) = @_;
    my $ProductType = $Servlet->object;
    my $Parent = $ProductType->folder('Attributes');
    my $Class = LoadClassByAlias('ProductAttribute');

    # create unique Alias from Name
    my $UniqueAlias = $Parent->uniqueChildAlias(Unicode2Alias($Servlet->form->value('Name')));

    # write alias to servlet
    my $hFormValues = $Servlet->form->values();
    $hFormValues->{'Alias'} = $UniqueAlias;
    $Servlet->form->setValues($hFormValues);

    eval {
        Transaction( sub {
            DE_EPAGES::Product::UI::ProductAttribute->NewObject( $Servlet, $Parent, $Class, {
                AttributeClass => $ProductType,
            } );
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code =~ /VariationFeatureLimitExceeded/){
            $Servlet->vars('VariationFeatureMaxValueReached',1);
            Warning($Error);
        } elsif ($Error->code  eq 'FeatureLimitExceeded'){
            $Servlet->vars('FeatureMaxValueReached', $Error->vars);
            Warning($Error);

        } else {
            Error($Error);
        }
    }

    return;
}

sub DeleteAttributes {
    my $self = shift;
    my ($Servlet) = @_;

    my $ProductType = $Servlet->object;
    my $Shop = $ProductType->getSite();

    my $Form = $Servlet->form;
    my $hValues = $Form->form($ProductType, 'ListedObjects');

    my @DeleteObjects;
    my @AttributeIDs = map { $_->{'ListObjectID'} } @{$hValues->{'ListObjectIDs'}};
    my $rAllAttributes = $Servlet->form->{'Values'}->{'ErrorIndex'};

    foreach my $Index ( 0 .. scalar(@AttributeIDs)-1 ) {
        if  ( scalar @{GetPKeySelectedVariationsByAttributeID($AttributeIDs[$Index])} and scalar @{GetPKeySelectPreDefAttributesByAttributeID($AttributeIDs[$Index])} ) {
           $Form->addFormError( {
               Reason => 'ProductsWithThisVariationAttributeExist',
               Index  => IndexOf($AttributeIDs[$Index],@$rAllAttributes),
               LoopName => 'ListObjectIDs',
               Form => 'ListedObjects',
               Name => 'ListObjectID',
           } );
        }
        push @DeleteObjects, $Shop->loadObject($AttributeIDs[$Index], 'ProductAttribute');
    }
    $Servlet->vars('DeleteObjects',\@DeleteObjects) if $Form->existsFormError;
    $Form->executeFormError;


    $self->SUPER::DeleteListedObjects( $Servlet );

    $ProductType->folder('Attributes')->renumberChildren;
    $ProductType->updatePageCache;
    return;
}

sub UnityDeleteAttributes {
    my $self = shift;
    my ($Servlet) = @_;

    my $ProductType = $Servlet->object;
    my $Shop = $ProductType->getSite();
    my $Form = $Servlet->form;
    my $hValues = $Form->form($ProductType, 'ListedObjects');
    my $Class = LoadClassByAlias('Object');

    # test if attribute is used for a variation
    my @DeleteIDs;
    my @DeleteObjects;
    my $rAllAttributes = $Form->{'Values'}->{'ErrorIndex'};
    my @AttributeIDs = map { $_->{'ListObjectID'} }
        grep { $Class->existsObject( $_->{'ListObjectID'} ) }
        @{$hValues->{'ListObjectIDs'}};
    foreach my $Index ( 0 .. scalar(@AttributeIDs)-1 ) {
        my $Name = $Shop->loadObject($AttributeIDs[$Index], 'Attribute')->get('Name');

        if  ( scalar @{GetPKeySelectedVariationsByAttributeID($AttributeIDs[$Index])} and scalar @{GetPKeySelectPreDefAttributesByAttributeID($AttributeIDs[$Index])} ) {
           $Form->addFormError( {
               Reason => 'ProductsWithThisVariationAttributeExist',
               Index  => IndexOf($AttributeIDs[$Index],@$rAllAttributes),
               LoopName => 'ListObjectIDs',
               Form => 'ListedObjects',
               Name => 'ListObjectID',
               AttributeName => $Name,
           } );
        } else {
            #create filtered id list (ignore ids used for variations)
            push @DeleteIDs, $AttributeIDs[$Index];
        }
        push @DeleteObjects, $Shop->loadObject($AttributeIDs[$Index], 'ProductAttribute');
    }

    # test permission
    my $aListObjects = $Servlet->object->getSite->loadObjects( \@DeleteIDs, $Class );
    my $TriggerFormError = 0;
    foreach my $Index ( 0 .. scalar(@$aListObjects)-1 ) {
        my $Object = $aListObjects->[$Index];
        unless ( $Servlet->hasPermission($self->testObjectDeleteListedObjects($Object), 'Delete')) {
            $Form->addFormError( {
                Reason => 'PermissionDenied',
                Index => $Index,
                LoopName => 'ListObjectIDs',
                Form => 'ListedObjects',
                Name => 'ListObjectID',
            } );
            $TriggerFormError = 1;
            next;
        }
        my $aWarnings = [];
        $Object->canDelete($aWarnings);
        $Form->mapFormError({
            'Form'     => 'ListedObjects',
            'Name'     => 'ListObjectID',
            'Index'    => $Index,
            'LoopName' => 'ListObjectIDs',
        }, $_) foreach @$aWarnings;
    }

    #trigger form error only for permission error
    if($TriggerFormError) {
        $Form->executeFormError;
    }

    # delete only if accepted if confirmation is used
    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        $Servlet->vars('DeleteObjects', [ map { $_->tleHash } @$aListObjects]);
        $Servlet->vars('DELETE',1);
        $Form->executeFormError;
        return;
    } else {
        # else reset delete
        $Servlet->vars('DELETE',0);
    }

    # now delete objects
    $self->deleteObjectListWrapper($aListObjects);

    $ProductType->folder('Attributes')->renumberChildren;
    $ProductType->updatePageCache;
    $Form->executeFormError;
    return;
}

sub SaveAttributes {
    my $self = shift;
    my ($Servlet) = @_;

    my $Alias = $Servlet->form->formValue('Alias');
    $self->NewAttribute( $Servlet ) if defined $Alias and $Alias =~ /\S/;

    my $ProductType = $Servlet->object;

    my @VisibleIDs = $Servlet->form->value('ListIsVisible');
    my @IsSearchFilterIDs = $Servlet->form->value('IsSearchFilter');

    $Servlet->form->executeFormError;

    # convert array to hash for faster access
    my %VisibleIDs = map { $_ => 1 } @VisibleIDs;
    my %IsSearchFilterIDs = map { $_ => 1 } @IsSearchFilterIDs;

    Transaction( sub {
        my $ahListItems = $self->listItems( $Servlet, $ProductType, 'ProductAttribute' );
        foreach my $hListItem (@$ahListItems) {
            my $Position = $hListItem->{'Position'};
            my $Attribute = $hListItem->{'Object'};
            my $IsVisible = $VisibleIDs{$Attribute->id} ? 1 : 0;
            my $IsSearchFilter = $IsSearchFilterIDs{$Attribute->id} ? 1 : 0;

            # security check - verify that the object is a child of given parent
            my $AttributeClass = $Attribute->get('AttributeClass');
            if( $AttributeClass->id != $ProductType->id ) {
                Error( 'WrongAttributeClass', {
                    AttributeClassID => $AttributeClass->id,
                    ProductTypeID => $ProductType->id,
                } );
            }
            my $HasFacetedSearch = $Attribute->class->existsAttribute( 'IsSearchFilter' );

            my $hVars = {'Position' => $Position,
                         'IsVisible' => $IsVisible,
                        };

            $hVars->{'IsSearchFilter'} = $IsSearchFilter if $HasFacetedSearch;

            $Attribute->set($hVars);
        }
    });

    $ProductType->folder('Attributes')->renumberChildren;
    $ProductType->updatePageCache;
    return;
}

sub UnitySaveAttributes {
    my $self = shift;
    my ($Servlet) = @_;
    my $Form = $Servlet->form;
    my $ProductType = $Servlet->object;
    $Form->form($ProductType, 'ProductTypeDetail');
    my $FormLanguageID = $Form->value('AltLanguageID');

    # test if name exists and create Attribute if it does (name is used to create alias)
    my $Name = $Form->formValue('Name');
    $self->UnityNewAttribute( $Servlet ) if defined $Name and $Name =~ /\S/;

    # save ProductType name
    $ProductType->set({'Name' => $Form->value('ProductTypeName')}, $FormLanguageID);

    $Form->executeFormError;

    # save position changes to list items
    Transaction( sub {
        my $ahListItems = $self->listItems( $Servlet, $ProductType, 'ProductAttribute' );
        foreach my $hListItem (@$ahListItems) {
            my $Position = $hListItem->{'Position'};
            my $Attribute = $hListItem->{'Object'};

            # security check - verify that the object is a child of given parent
            my $AttributeClass = $Attribute->get('AttributeClass');
            if( $AttributeClass->id != $ProductType->id ) {
                Error( 'WrongAttributeClass', {
                    AttributeClassID => $AttributeClass->id,
                    ProductTypeID => $ProductType->id,
                } );
            }

            $Attribute->set({'Position' => $Position});
        }
    });

    $ProductType->folder('Attributes')->renumberChildren;
    $ProductType->updatePageCache;
    return;
}

sub SaveDesign {
    my $self = shift;
    my ($Servlet) = @_;
    my $Object = $Servlet->object;

    my $PageType = LoadPageTypeByAlias( 'SF-Product' );

    foreach my $TemplateFolder (@{$PageType->templateTypeFolders}){
        my $TemplateName = $TemplateFolder->alias;
        my $TemplateTypeName = $Servlet->form->value($TemplateName);
        next unless defined $TemplateTypeName;

        # show select box at base template
        if ($TemplateName eq 'Content-Variation') {
            my $VariationTemplateType = $PageType->templateTypeFolder('Content-Base-SelectVariation')->child(
                $TemplateTypeName eq 'VariationSelectBox' ? 'SelectBox' : 'NoSelection'
            );
            $PageType->setTemplateType( $Object, 'Content-Base-SelectVariation', $VariationTemplateType->alias );

            # special Content-Variation treatment for new templates (EPG-17552), this will be removed in 6.15.0
            my $VariationTemplateTypeNew = $PageType->templateTypeFolder('Content-PriceBox-SelectVariation')->child(
                $TemplateTypeName eq 'VariationSelectBox' ? 'SelectBox' : 'NoSelection'
            );
            $PageType->setTemplateType( $Object, 'Content-PriceBox-SelectVariation', $VariationTemplateTypeNew->alias );

            # use not visible Content-Variation-New template type folder
            $PageType->setTemplateType( $Object, 'Content-Variation-New', $TemplateTypeName.'_New' );
        }
        elsif ($TemplateName eq 'Content-Bundles') {
            # special Content-Bundles treatment for new templates (EPG-17552), this will be removed in 6.15.0
            # use not visible Content-Bundles-New template type folder
            $PageType->setTemplateType( $Object, 'Content-Bundles-New', $TemplateTypeName.'_New' );
        }

        $PageType->setTemplateType( $Object, $TemplateName, $TemplateTypeName );
    }
    $Object->updatePageCache;
    return;
}

sub View {
    my $self = shift;
    my ($Servlet) = @_;
    my $Object = $Servlet->object;

    $Servlet->vars('ProductTypeName', $Object->get('Name'));
}

sub UnityView {
    my $self = shift;
    my ($Servlet) = @_;

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    my $LocaleID   = $Servlet->locale();
    my $Locale     = DE_EPAGES::Object::API::Object::Locale->load($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');

    my $AltLanguageID = $Servlet->params('AltLanguageID');
    my $Object = $Servlet->object;

    $Servlet->vars('ProductTypeName', $Object->get('Name', $AltLanguageID // $LanguageID));
}

1;
