#========================================================================================
# package      DE_EPAGES::Saferpay2::t::unit::BasketOrder
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Saferpay2::UI::BasketOrder
#========================================================================================
package DE_EPAGES::Saferpay2::t::unit::BasketOrder;
use base DE_EPAGES::Core::API::Test;

use Test::Exception;
use Test::More;
use Test::Mockify;
use Test::Mockify::Sut;
use Test::Mockify::Matcher qw ( Object HashRef String );
use Test::Mockify::Verify qw ( GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;

use strict;

use constant SAFERPAY_INITIALIZE_TOKEN => 'KJH765765HGFHF';

sub test {
    my $self = shift;

    $self->test_ViewPaymentSaferpay2Successful();
    $self->test_ViewPaymentSaferpay2Unsuccessful();

    done_testing();
}


sub test_ViewPaymentSaferpay2Successful {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $Order = $self->_createOrder();
    my $MockedServlet = $self->_createServlet($Order);
    my $MockedSaferpayService = $self->_createSaferpayService($self->_createSuccessfulInitializePaymentResponse());

    my $SUT = $self->_createSUT($MockedSaferpayService);

    $SUT->ViewPaymentSaferpay2($MockedServlet);

    my $aConfirmInitializeParameters = GetParametersFromMockifyCall($MockedSaferpayService , 'InitializePaymentPage');
    my $aExpectedParameters = [$Order, 'http://webhook.url', 'http://redirect.url'];

    is_deeply($aConfirmInitializeParameters, $aExpectedParameters, "$SubTestName - SaferpayService InitializePaymentPage parameters ok.");

    is($MockedServlet->vars->{'SaferpayRedirectUrl'}, 'http://saferpay_checkout.url', "$SubTestName - SaferpayCheckoutUrl ok.");

    my $MockedLineItemPayment = $Order->get('LineItemContainer')->get('Payment');

    is($MockedLineItemPayment->get('TransID2'), SAFERPAY_INITIALIZE_TOKEN, "$SubTestName - Token is correctly saved in LineItemPayment.");
    is($MockedLineItemPayment->get('IsTestTransaction'), 1, "$SubTestName - IsTestTransaction is correctly saved in LineItemPayment.");
}

sub test_ViewPaymentSaferpay2Unsuccessful {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $Order = $self->_createOrder();
    my $MockedServlet = $self->_createServlet($Order);

    my $MockedSaferpayService = $self->_createSaferpayService($self->_createUnsuccessfulInitializePaymentResponse());
    my $SUT = $self->_createSUT($MockedSaferpayService);

    $SUT->ViewPaymentSaferpay2($MockedServlet);
    is($MockedServlet->vars->{'SaferpayPaymentFailed'}, 1, "$SubTestName - SaferpayPaymentFailed ok.");
    is($MockedServlet->vars->{'SaferpayPaymentFailedReason'}, 'INTERNAL_SERVER_ERROR', "$SubTestName - SaferpayPaymentFailedReason ok.");

    my $MockedLineItemPayment = $Order->get('LineItemContainer')->get('Payment');

    is($MockedLineItemPayment->get('TransID2'), undef, "$SubTestName - Token is undefined in LineItemPayment.");
    is($MockedLineItemPayment->get('IsTestTransaction'), 1, "$SubTestName - IsTestTransaction is correctly saved in LineItemPayment.");
}


sub _createServlet {
    my $self = shift;
    my ( $Order ) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Order);
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('redirect', undef, ['string']);

    return $MockifyServlet->getMockObject();
}


sub _createOrder {
    my $self = shift;

    my $Shop = $self->_createShop();
    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer(),
        'Shop'              => $Shop,
        'Alias'             => '42',
        'ID'                => 654321
    };

    my $MockifyCustomerOrder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes
    );

    $MockifyCustomerOrder->addMockWithReturnValue('getSite', $hAttributes->{'Shop'});
    return $MockifyCustomerOrder->getMockObject();
}


sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {
        'Payment'    => $self->_createLineItemPaymentSaferpay2(),
        'CurrencyID' => 'EUR',
        'GrandTotal' => '10'
    };
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes
    );

    return $MockifyContainer->getMockObject();
}


sub _createLineItemPaymentSaferpay2 {
    my $self = shift;

    my $hAttributes = {
        'PaymentMethod' => $self->_createPaymentMethod(),
        'GUID'          => 123456,
        'TransID2' => undef
    };
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Saferpay2::API::Object::LineItemPaymentSaferpay2', $hAttributes
    );

    $MockifyContainer->addMockWithReturnValueAndParameterCheck(
        'updateTransactionStatus', undef, ['string', 'string', 'string']
    );

    return $MockifyContainer->getMockObject();
}


sub _createPaymentMethod {
    my $self = shift;

    my $hAttributes = {
        CustomerId       => '123456',
        TerminalId       => '654321',
        MerchantName     => 'username',
        MerchantPassword => 'password',
        IsInTestMode     => 1
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Saferpay2::API::Object::PaymentMethodSaferpay2', $hAttributes
    );

    return $MockifyPaymentMethod->getMockObject();
}


sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'NameOrAlias' => '23',
        'instanceOf'  => 'Shop',
        'LocaleID'    => '1',
        'ID'          => '1'
    };

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );
    return $MockifyShop->getMockObject();

}


sub _createSUT {
    my $self = shift;
    my ( $SaferpayService ) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::Saferpay2::UI::BasketOrder');
    $MockifySut->mock('_LogPayment')->when(String(), HashRef() )->thenReturnUndef();
    $MockifySut->mock('_buildReturnURL')->when(Object(), Object(), Object('DE_EPAGES::Saferpay2::API::Object::LineItemPaymentSaferpay2'), HashRef({'ChangeAction' => 'PaymentCallbackSaferpay2'}) )->thenReturn('http://webhook.url');
    $MockifySut->mock('_buildReturnURL')->when(Object(), Object(), Object('DE_EPAGES::Saferpay2::API::Object::LineItemPaymentSaferpay2'), HashRef({'ChangeAction' => 'PaymentBacklinkSaferpay2'}) )->thenReturn('http://redirect.url');
    $MockifySut->mockStatic('DE_EPAGES::Saferpay2::API::SaferpayService::new')->when( Object() )->thenReturn($SaferpayService);

    return $MockifySut->getMockObject();
}


sub _createSaferpayService() {
    my $self = shift;
    my ( $response )= @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::Saferpay2::API::SaferpayService');
    $Mockify->mock('InitializePaymentPage')->when( Object(), String(), String() )->thenReturn($response);
    return $Mockify->getMockObject();

}


sub _createSuccessfulInitializePaymentResponse() {
    my $self = shift;
    my $hResponse = {
        RedirectUrl    => 'http://saferpay_checkout.url',
        Token => SAFERPAY_INITIALIZE_TOKEN
    };
    return $hResponse;
}


sub _createUnsuccessfulInitializePaymentResponse() {
    my $self = shift;
    my $hResponse = {
        Behavior     => 'ABORT',
        ErrorMessage => 'Internal Server Error',
        ErrorName => 'INTERNAL_SERVER_ERROR'
    };
    return $hResponse;
}


#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();
