#========================================================================================
# package      DE_EPAGES::Saferpay2::t::unit::SaferpayService.t
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Saferpay2::API::SaferpayService
#========================================================================================
package DE_EPAGES::Saferpay2::t::unit::SaferpayService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Saferpay2::API::Constants qw ( SUPPORTED_PAYMENT_METHODS );
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use JSON::MaybeXS qw ( encode_json decode_json );
use MIME::Base64;
use Test::Exception;
use Test::Mockify::Verify qw ( GetParametersFromMockifyCall );
use Test::Mockify::Sut;
use Test::Mockify::Matcher qw ( String HashRef Object ArrayRef );
use Test::More;

use strict;

use constant SAFERPAY_BASE_URL => "https://saferpay2.api/";
use constant EXPECTED_RESPONSE => '{"status": "200 OK"}';
use constant SAFERPAY_TOKEN => '123';
use constant SOME_MERCHANT_NAME => 'SomeMerchantName';
use constant SOME_MERCHANT_PASSWORD => 'SomeMerchantPassword';
use constant SOME_CUSTOMER_ID => 'SomeCustomerId';
use constant SOME_TERMINAL_ID => 'SomeTerminalId';


sub test {
    my $self = shift;

    $self->test_SaferpayServiceInitialization();
    $self->test_InitializePaymentPage();
    $self->test_FormatFloatForSaferpay();
    $self->test_DoAssertCall();
    $self->test_DoAssertCallCustomValues();
    done_testing();
}


sub test_SaferpayServiceInitialization {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = $self->_createUserAgent();
    my $MockedOrder = $self->_createOrder();
    my $MockedLineItemContainer = $MockedOrder->get('LineItemContainer');
    my $MockedLineItemPayment = $MockedLineItemContainer->get('Payment');
    my $MockedPaymentMethod = $MockedLineItemPayment->get('PaymentMethod');

    my $SUT;
    lives_ok( sub { $SUT = $self->_createSUT($MockedLineItemPayment, $MockedUserAgent, undef) }, "$SubTestName - Service initialization works." );
    is($SUT->{'MerchantName'}, $MockedPaymentMethod->get('MerchantName'), "$SubTestName - Correct MerchantName defined for service.");
    is($SUT->{'MerchantPassword'}, $MockedPaymentMethod->get('MerchantPassword'), "$SubTestName - Correct MerchantPassword defined for service.");
    is($SUT->{'CustomerId'}, $MockedPaymentMethod->get('CustomerId'), "$SubTestName - Correct CustomerId defined for service.");
    is($SUT->{'TerminalId'}, $MockedPaymentMethod->get('TerminalId'), "$SubTestName - Correct TerminalId defined for service.");

    my $CustomValues = $self->_createCustomValues();

    lives_ok( sub { $SUT = $self->_createSUT($MockedLineItemPayment, $MockedUserAgent, $CustomValues) }, "$SubTestName - Service initialization with custom values works." );
    is($SUT->{'MerchantName'}, SOME_MERCHANT_NAME, "$SubTestName - Correct custom value MerchantName defined for service.");
    is($SUT->{'MerchantPassword'}, SOME_MERCHANT_PASSWORD, "$SubTestName - Correct custom value MerchantPassword defined for service.");
    is($SUT->{'CustomerId'}, SOME_CUSTOMER_ID, "$SubTestName - Correct custom value CustomerId defined for service.");
    is($SUT->{'TerminalId'}, SOME_TERMINAL_ID, "$SubTestName - Correct custom value TerminalId defined for service.");
}

sub test_InitializePaymentPage {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = $self->_createUserAgent();
    my $MockedOrder = $self->_createOrder();
    my $MockedLineItemContainer = $MockedOrder->get('LineItemContainer');
    my $MockedLineItemPayment = $MockedLineItemContainer->get('Payment');

    my $SUT = $self->_createSUT($MockedLineItemPayment, $MockedUserAgent);
    my $RedirectUrl = 'http://redirect.url';
    my $WebhookUrl = 'http://webhook.url';

    my $InitializePaymentPageResponse;
    lives_ok( sub { $InitializePaymentPageResponse = $SUT->InitializePaymentPage($MockedOrder, $WebhookUrl, $RedirectUrl) }, "$SubTestName - Initialize payment page ok." );
    is_deeply($InitializePaymentPageResponse, decode_json(EXPECTED_RESPONSE), "$SubTestName - Response is ok.");

    my $ExpectedRequestBody = {
        RequestHeader => {
            SpecVersion    => '1.18',
            CustomerId     => '123456',
            RequestId      => 'ABC123456',
            RetryIndicator => 0
        },
        TerminalId    => '654321',
        Payment       => {
            Amount      => {
                Value        => '1000',
                CurrencyCode => 'EUR'
            },
            OrderId     => '42',
            Description => '23'
        },
        PaymentMethods => SUPPORTED_PAYMENT_METHODS,
        ReturnUrls    => {
            Success => $RedirectUrl,
            Fail    => $RedirectUrl
        },
        Notification  => {
            NotifyUrl => $WebhookUrl
        },
        RequestURL    => SAFERPAY_BASE_URL."Payment/v1/PaymentPage/Initialize"
    };

    my $ActualRequestObject = GetParametersFromMockifyCall($MockedUserAgent, 'request')->[0];
    my $ExpectedAuthHeader = 'Basic '.encode_base64('username:password', '');
    is($ActualRequestObject->decoded_content(), encode_json($ExpectedRequestBody), "$SubTestName - Request body is ok.");
    is($ActualRequestObject->header('Authorization'), $ExpectedAuthHeader, "$SubTestName - Authorization header ok.");

}

sub test_FormatFloatForSaferpay {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedLineItemPayment = $self->_createLineItemPaymentSaferpay2();
    my $MockedUserAgent = $self->_createUserAgent();
    my $SUT = $self->_createSUT($MockedLineItemPayment, $MockedUserAgent);

    my $Amount = sprintf("%.2f", rand(1000000));
    my $Currencyid = 'EUR';
    my $ExpectedResult;

    is($SUT->_FormatFloatForSaferpay(''.$Amount, $Currencyid), ''.($ExpectedResult = $Amount * 100), "$SubTestName - in was $Amount, out was $ExpectedResult,  Format float is ok.");

    $Amount = 0.00;
    is($SUT->_FormatFloatForSaferpay(''.$Amount, $Currencyid), ''.($ExpectedResult = $Amount * 100), "$SubTestName - in was $Amount, out was $ExpectedResult,  Format float is ok.");

    $Amount = 99999999.99;
    is($SUT->_FormatFloatForSaferpay(''.$Amount, $Currencyid), ''.($ExpectedResult = $Amount * 100), "$SubTestName - in was $Amount, out was $ExpectedResult,  Format float is ok.");
}

sub test_DoAssertCall {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedLineItemPayment = $self->_createLineItemPaymentSaferpay2();
    my $MockedUserAgent = $self->_createUserAgent();

    my $SUT = $self->_createSUT($MockedLineItemPayment, $MockedUserAgent);
    my $DoAssertCallResponse;
    lives_ok( sub { $DoAssertCallResponse = $SUT->DoAssertCall(SAFERPAY_TOKEN) }, "$SubTestName - Do Assert call ok." );
    is_deeply($DoAssertCallResponse, decode_json(EXPECTED_RESPONSE), "$SubTestName - Response is ok.");

    my $ExpectedRequestBody = {
        RequestHeader =>  {
            SpecVersion => "1.18",
            CustomerId => "123456",
            RequestId => "ABC123456",
            RetryIndicator => 0
        },
        Token => SAFERPAY_TOKEN,
        RequestURL    => SAFERPAY_BASE_URL."Payment/v1/PaymentPage/Assert"
    };
    my $ActualRequestObject = GetParametersFromMockifyCall($MockedUserAgent, 'request')->[0];
    my $ExpectedAuthHeader = 'Basic '.encode_base64('username:password', '');
    is($ActualRequestObject->header('Authorization'), $ExpectedAuthHeader, "$SubTestName - Authorization header ok.");
    is($ActualRequestObject->decoded_content(), encode_json($ExpectedRequestBody), "$SubTestName - Request body is ok.");
}

sub test_DoAssertCallCustomValues {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedLineItemPayment = $self->_createLineItemPaymentSaferpay2();
    my $MockedUserAgent = $self->_createUserAgent();

    my $hValues = $self->_createCustomValues();

    my $SUT = $self->_createSUT($MockedLineItemPayment, $MockedUserAgent, $hValues);
    my $DoAssertCallResponse;
    lives_ok( sub { $DoAssertCallResponse = $SUT->DoAssertCall(SAFERPAY_TOKEN) }, "$SubTestName - Do Assert call ok." );
    is_deeply($DoAssertCallResponse, decode_json(EXPECTED_RESPONSE), "$SubTestName - Response is ok.");

    my $ExpectedRequestBody = {
        RequestHeader =>  {
            SpecVersion => "1.18",
            CustomerId => SOME_CUSTOMER_ID,
            RequestId => "ABC123456",
            RetryIndicator => 0
        },
        Token => SAFERPAY_TOKEN,
        RequestURL    => SAFERPAY_BASE_URL."Payment/v1/PaymentPage/Assert"
    };
    my $ActualRequestObject = GetParametersFromMockifyCall($MockedUserAgent, 'request')->[0];
    my $ExpectedAuthHeader = 'Basic '.encode_base64(SOME_MERCHANT_NAME.':'.SOME_MERCHANT_PASSWORD, '');
    is($ActualRequestObject->header('Authorization'), $ExpectedAuthHeader, "$SubTestName - Authorization header ok.");
    is($ActualRequestObject->decoded_content(), encode_json($ExpectedRequestBody), "$SubTestName - Request body is ok.");
}

sub _createOrder {
    my $self = shift;

    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer(),
        'Shop'              => $self->_createShop(),
        'Alias'             => '42',
    };
    my $MockifyCustomerOrder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes
    );
    $MockifyCustomerOrder->addMockWithReturnValue('getSite', $hAttributes->{'Shop'});

    return $MockifyCustomerOrder->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'NameOrAlias' => '23'
    };

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );
    return $MockifyShop->getMockObject();

}


sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {
        'Payment'    => $self->_createLineItemPaymentSaferpay2(),
        'CurrencyID' => 'EUR',
        'GrandTotal' => '10'
    };
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes
    );

    return $MockifyContainer->getMockObject();
}


sub _createLineItemPaymentSaferpay2 {
    my $self = shift;

    my $hAttributes = {
        'PaymentMethod'     => $self->_createPaymentMethod(),
        'TransID2'          => '123',
        'IsTestTransaction' => 1
    };
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Saferpay2::API::Object::LineItemPaymentSaferpay2', $hAttributes
    );

    return $MockifyContainer->getMockObject();
}


sub _createPaymentMethod {
    my $self = shift;

    my $hAttributes = {
        CustomerId       => '123456',
        TerminalId       => '654321',
        MerchantName     => 'username',
        MerchantPassword => 'password',
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Saferpay2::API::Object::PaymentMethodSaferpay2', $hAttributes
    );

    return $MockifyPaymentMethod->getMockObject();
}


sub _createSystem {
    my $self = shift;

    my $hAttributes = {
        Saferpay2_WebserviceEndpointURL_test => SAFERPAY_BASE_URL
    };
    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Site', $hAttributes
    );

    return $MockifySystem->getMockObject();
}


sub _createSUT {
    my $self = shift;
    my ( $LineItemPayment, $UserAgent, $hValues ) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::Saferpay2::API::SaferpayService', [$LineItemPayment, $hValues]);
    $MockifySut->mockImported('DE_EPAGES::Object::API::Factory', 'LoadRootObject')->when()->thenReturn($self->_createSystem());
    $MockifySut->mockImported('DE_EPAGES::Object::API::Table::Currency', 'GetInfoCurrency')->when(String())->thenReturn({ MinorUnit=>2 });
    $MockifySut->mock('_LogPayment')->when(String(), HashRef())->thenReturnUndef();
    $MockifySut->mockStatic('LWP::UserAgent::new')->when()->thenReturn($UserAgent);
    $MockifySut->spy('_doPostRequest')->when(String(), ArrayRef(), String());
    $MockifySut->mockImported('DE_EPAGES::Core::API::PerlTools', 'GenerateGUID')->when()->thenReturn('ABC123456');
    return $MockifySut->getMockObject();
}


sub _createUserAgent {
    my $self = shift;

    my $MockifyUA = Test::Mockify->new('LWP::UserAgent');
    $MockifyUA->mock('request')->when(Object())->thenReturn($self->_createResponse());
    return $MockifyUA->getMockObject();
}


sub _createResponse {
    my $self = shift;

    my $MockifyResponse = Test::Mockify->new('HTTP::Response');
    $MockifyResponse->mock('code')->when()->thenReturn('200');
    $MockifyResponse->mock('decoded_content')->when()->thenReturn(EXPECTED_RESPONSE);
    return $MockifyResponse->getMockObject();

}

sub _createCustomValues {
    my $self = shift;

    return {
        MerchantName     => SOME_MERCHANT_NAME,
        MerchantPassword => SOME_MERCHANT_PASSWORD,
        CustomerId       => SOME_CUSTOMER_ID,
        TerminalId       => SOME_TERMINAL_ID
    };
}


#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();
