#========================================================================================
# package      DE_EPAGES::Shop::Hooks::Shop
# state        public
# description  Imports default shop data into a new shop when it is created.
#========================================================================================
package DE_EPAGES::Shop::Hooks::Shop;

use strict;
use JSON;
use DE_EPAGES::Core::API::EpagesConfig qw ( EpagesConfig );
use DE_EPAGES::Core::API::PerlTools qw( GetCurrentDateTime );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::OAuthTokenClient::API::OAuth2::OAuth2CredentialsClient;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Shop::API::Cartridge;
use DE_EPAGES::Shop::API::Constants qw( UNITY_STOREFRONT_APP_NAME UNITY_STOREFRONT_APP_TYPE );
use DE_EPAGES::Object::API::Domain qw ( GetUnitySubDomain );

#========================================================================================
# function     OnInsertShop
#----------------------------------------------------------------------------------------
# syntax       OnInsertShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Imports default shop data into a new shop when it is created.
#               (Hook OBJ_InsertShop)
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                   <li>Object - shop - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    my $Cartridge = DE_EPAGES::Shop::API::Cartridge->new;
    ImportObjects($Cartridge->xmlDirectory().'/DefaultShop.xml', $Shop->pathString);

}

#========================================================================================
# function     GetShopFeatureCurrentValue
#----------------------------------------------------------------------------------------
# syntax       GetShopFeatureCurrentValue( $hParams );
#----------------------------------------------------------------------------------------
# description  Dynami Values of current features i.e. WebServiceFetures with date reset
#               (Hook OBJ_GetShopFeatureCurrentValue)
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                   <li>Object - shop - object
#                   <li>Name - feature name - string
#                   <li>FeatureID - feature id - integer
#               </ul> | ref.hash.*
#----------------------------------------------------------------------------------------
# return      $hParams | hook parameters
#               <ul>
#                   <li>Value - current value - integer
#               </ul> | ref.hash.*
#========================================================================================
sub GetShopFeatureCurrentValue {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    my $FeatureAlias = $hParams->{'Name'};

    if ( $FeatureAlias =~ /WebServices./ ) {
        # compare current date with last WS date, reset feature count if not equal
        my $CurrentDate = GetCurrentDateTime();
        my $DateAttributeName = $FeatureAlias.'Date';
        if ( $Shop->class->existsAttribute($DateAttributeName) ) {
            my $LastDate = $Shop->get( $DateAttributeName );
            if ( !$LastDate ||
                  $LastDate->day    != $CurrentDate->day ||
                  $LastDate->month  != $CurrentDate->month )
            {
                $Shop->set( {$DateAttributeName => $CurrentDate} );
                $Shop->featureCurrentValue( $FeatureAlias, 0 );
                $hParams->{'Value'} = 0;
            }
        }
    }

    return 0;
}

#========================================================================================
# function     GetShopFeatureMaxValue
#----------------------------------------------------------------------------------------
# syntax       GetShopFeatureMaxValue( $hParams );
#----------------------------------------------------------------------------------------
# description  Returns the max value of a features
#               (Hook OBJ_GetShopFeatureMaxValue)
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                   <li>Object - shop - object
#                   <li>Name - feature name - string
#               </ul> | ref.hash.*
#----------------------------------------------------------------------------------------
# return      $hParams | hook parameters
#               <ul>
#                   <li>Value - current value - integer
#               </ul> | ref.hash.*
#========================================================================================
sub GetShopFeatureMaxValue {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    my $FeatureAlias = $hParams->{'Name'};

    $hParams->{'Value'} = $Shop->featureMaxValue($FeatureAlias);
    return $hParams->{'Value'};
}

#========================================================================================
# function     OnBeforePatchUnityShop
#----------------------------------------------------------------------------------------
# syntax       OnBeforePatchUnityShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook API_BeforePatchUnityShop.
#               Sets the shop content that should be patched to the Unity database.
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                 <li>Object  - shop                              - object
#                 <li>Values  - new shop values                   - ref.hash
#                 <li>Content - content to patch to the unity API - ref.hash
#                 <li>LanguageID - language ID                    - int
#               </ul>
#========================================================================================
sub OnBeforePatchUnityShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    my $hValues = $hParams->{'Values'};
    my $hContent = $hParams->{'Content'};
    my $LanguageID = $hParams->{'LanguageID'};

    # add values to $hContent for updating localized attributes in unity
    if (defined $LanguageID) {

        if (exists $hValues->{'Name'}) {
            $hContent->{'title'} = $hValues->{'Name'} // '';
        }
    }

    # add values to $hContent for updating non localized attributes in unity
    if (   exists $hValues->{'ClosedByMerchant'}
        or exists $hValues->{'ClosedByProvider'}
        or exists $hValues->{'ClosedByProviderTemporarily'}
        or exists $hValues->{'IsClosed'}) {
        if ($hValues->{'ClosedByMerchant'} or $hValues->{'ClosedByProvider'} or $hValues->{'ClosedByProviderTemporarily'} or $hValues->{'IsClosed'}) {
            $hContent->{'isClosed'} = JSON::true;
        }
        else {
            my $hClosedSettings = $Shop->get(['ClosedByMerchant', 'ClosedByProvider', 'ClosedByProviderTemporarily', 'IsClosed']);
            $hContent->{'isClosed'} = JSON::false unless ($hClosedSettings->{'ClosedByMerchant'}
                or $hClosedSettings->{'ClosedByProvider'} or $hClosedSettings->{'ClosedByProviderTemporarily'} or $hClosedSettings->{'IsClosed'});
        }
    }

    if (exists $hValues->{'Logo'}) {
        if($Shop->get('UseUnityShopLogo')){
            delete $hContent->{'logoUrl'};
        }else {
            $hContent->{'logoUrl'} = $Shop->getLogoUrl($hValues->{'Logo'});
        }
    }
    if (exists $hValues->{'FavIcon'}) {
        $hContent->{'faviconUrl'} = $Shop->getFavIconUrl($hValues->{'FavIcon'});
    }
    # Patch URL values in case the SSL or domain setting of the shop changes
    if ((exists $hValues->{'HasSSLCertificate'}) or ((exists $hValues->{'DomainName'}))) {
        if (exists $hValues->{'DomainName'} && $Shop->get('HasSSLCertificate')) {
            # only send domain to Unity in case we have an SSL certificate (UNITY-3778)
            $hContent->{'domain'} = $hValues->{'DomainName'};
        }
        if (exists $hValues->{'HasSSLCertificate'}) {
            if ($hValues->{'HasSSLCertificate'}) {
                my $DomainName = $Shop->get('DomainName');
                # send domain to Unity in case we add an SSL certificate (UNITY-3778)
                $hContent->{'domain'} = $DomainName if defined $DomainName;
            } else {
                # delete domain from Unity database in case of no SSL certificate
                $hContent->{'domain'} = undef;
            }
        }
        $hContent->{'mboBaseUrl'} = $Shop->get('WebUrlAdminSSL');
    }

    return;
}

sub OnAfterPatchUnityShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    my $hValues = $hParams->{'Values'};

    if (exists $hValues->{'Logo'}) {
        return if $Shop->get('UseUnityShopLogo');
        $Shop->set({'UseUnityShopLogo' => 1});
    }
}

#========================================================================================
# function     OnBeforeCreateUnityShop
#----------------------------------------------------------------------------------------
# syntax       OnBeforeCreateUnityShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook API_OnBeforeCreateUnityShop.
#               Sets the shop content for the shop creation on the Unity database.
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                 <li>Object  - shop                             - object
#                 <li>Content - content to send to the unity API - ref.hash
#               </ul>
#========================================================================================
sub OnBeforeCreateUnityShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    my $hContent = $hParams->{'Content'};

    # import shop notifications
    my $Cartridge = DE_EPAGES::Shop::API::Cartridge->new;
    ImportObjects($Cartridge->xmlDirectory().'/ShopNotifications.xml', $Shop->pathString);

    # import uneditable and prefilled attributes' translations
    my @ShopLanguageIDs = map { $_->{'LanguageID'} } @{$Shop->get('Languages')};
    foreach my $ShopLanguageID (@ShopLanguageIDs) {
        $Shop->importUneditableAttributesUnityTranslations($ShopLanguageID);
        $Shop->importPrefilledAttributesUnityTranslations($ShopLanguageID);
    }

    my $System = LoadRootObject();
    my $hInfoShop = $Shop->get([
        'DomainName',
        'FavIcon',
        'GUID',
        'HasSSLCertificate',
        'LocaleID',
        'Logo',
        'NextSteps',
        'WebServerScriptNamePart',
        'WebUrlAdminSSL',
    ]);

    # initialize first steps
    my $hItems = $hInfoShop->{'NextSteps'} // {};
    $hItems->{'ViewShop'} = 0;
    $hItems->{'LaunchShop'} = 0;
    # close every new created Unity shop per default
    $Shop->set({
        'ClosedByMerchant' => 1,
        'NextSteps' => $hItems
    });

    $hContent->{'isClosed'}   = JSON::true;
    $hContent->{'title'}      = '';
    $hContent->{'locale'}     = $hInfoShop->{'LocaleID'};
    $hContent->{'mboBaseUrl'} = $hInfoShop->{'WebUrlAdminSSL'};
    $hContent->{'logoUrl'}    = $Shop->getLogoUrl($hInfoShop->{'Logo'});
    $hContent->{'faviconUrl'} = $Shop->getFavIconUrl($hInfoShop->{'FavIcon'});
    $hContent->{'domain'}     = $hInfoShop->{'DomainName'} if $hInfoShop->{'HasSSLCertificate'};
    $hContent->{'jrest'}      = {
        'baseUrl'  => $System->get('UnityBaseURL'),
        'publicId' => $hInfoShop->{'WebServerScriptNamePart'}
    };

    if (EpagesConfig->get('DE_EPAGES::Object', 'UnityUseAuth', 1)) {
        my $Client = DE_EPAGES::OAuthTokenClient::API::OAuth2::OAuth2CredentialsClient->create;
        my $Credentials = $Client->createAndSaveCredentials($hInfoShop->{'GUID'}, {
            'name' => UNITY_STOREFRONT_APP_NAME,
            'type' => UNITY_STOREFRONT_APP_TYPE,
        });

        $hContent->{'jrest'}->{'oauth'} = {
            'version'     => '2.0',
            'credentials' => {
                'clientId'           => $Credentials->clientId,
                'clientSecret'       => $Credentials->clientSecret,
                'accessToken'        => $Credentials->accessToken
            }
        };
    }
}

#========================================================================================
# function     OnAfterUpdateShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateShop( $hParams );
#----------------------------------------------------------------------------------------
# description  if shop is updated, check if all next steps are completed
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                 <li>Object        - shop object          - object
#                 <li>ObjectID      - shop object id            - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};

    if ($Shop->get('IsUnityRegistered')) {
        my $hValues = $hParams->{'Values'};
        # check if all next steps are completed for Unity Shops
        if (exists $hValues->{'NextSteps'}) {
            my $hItems = $Shop->get('NextSteps') // {};
            my $NextStepsCompleted = 1;
            foreach my $Value (values %$hItems) {
                $NextStepsCompleted = 0 unless $Value;
            }
            $Shop->set({ 'NextStepsCompleted' => $NextStepsCompleted });
        }

        # change the status of the shop closed notification for Unity shops
        if (exists $hValues->{'ClosedByMerchant'}) {
            my $ShopClosedNotification = $Shop->folder('ShopNotifications')->child('ShopClosed');
            if ($hValues->{'ClosedByMerchant'}) {
                $ShopClosedNotification->activate();
            } else {
                $ShopClosedNotification->deactivate();
                # set first step
                my $hItems = $Shop->get('NextSteps') // {};
                unless ($hItems->{'LaunchShop'}) {
                    $hItems->{'LaunchShop'} = 1;
                    $Shop->set({'NextSteps' => $hItems});
                }
            }
        }
    }

    return;
}

#========================================================================================
# function     OnGetShopLogo
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnGetShopLogo( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook OBJ_GetShop.
#               Sets the URL of the shop logo in hParams->{'Value'}. This value will be
#               set to "null", if the shop name should be displayed (indicated by
#               'shouldHideLogo')
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                 <li>LanguageID    | int
#                 <li>aNames        | array
#                 <li>Shop Object   | object
#               </ul> | ref.hash.*
#========================================================================================
sub OnGetShopLogo {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};

    # do the next steps only for attribute Logo and for now shops
    return unless $hParams->{'Names'} eq 'Logo';

    my $hShopParams = $Shop->get( [qw(UseUnityShopLogo IsUnityRegistered InitialShopWizardStartup WebServerScriptNamePart)]);

    # we not get the logo from epages ui if we have a running shop wizard or not an now shop
    # and not set the attribute UseUnityShopLogo (this attr. is set after finishing the shop wizard)
    return if $hShopParams->{'InitialShopWizardStartup'};
    return unless $hShopParams->{'UseUnityShopLogo'};
    return unless $hShopParams->{'IsUnityRegistered'};

    my $ShopContent;
    eval {
        $ShopContent = $Shop->sendUnityRequest('GET', '/shop');
    };
    if (ExistsError()) {
        # Log the error if any, but do not throw it to the frontend
        my $Error = GetError();
        LogError($Error);
        return;
    }

    # return, if no logo has been uploaded in the editor
    return unless defined($ShopContent->{'logoUrl'});

    # if we have a remote image (e.g. the image is in epages6) we do not add the shop name part....
    my $LogoUrl = $ShopContent->{'logoUrl'};
    unless ($LogoUrl =~ /image\?remote=/) {
        $LogoUrl .= '&shop=' . GetUnitySubDomain($hShopParams->{'WebServerScriptNamePart'});
    }

    # set Value to "null" if logo should be hidden
    # NOTE: Value can not be set to undef or else logo will be overwritten later
    $hParams->{'Value'} = $ShopContent->{'shouldHideLogo'} ? "null" : $LogoUrl;

    return;
}

1;
