#==============================================================================
# package      DE_EPAGES::ShopTransfer::API::Export
# state        public
# description  Exporter for the shop tranfer
#==============================================================================
package DE_EPAGES::ShopTransfer::API::Export;
use base Exporter;

use strict;

use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::File qw (ExistsFile DeleteFile);
use DE_EPAGES::Core::API::PerlTools qw (UsePackage);
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::ShopTransfer::API::BCP qw (BCPOutStatement);

sub ErrorDefinition {
    return {
        'TableWithoutPrimaryKey' => 'Table #Table does not have a primary key.',
    };
}

sub new {
    my $class = shift;

    my $DBIPackage = GetCurrentDBHandle()->driverName =~ /mysql/i
        ? 'DE_EPAGES::ShopTransfer::API::DBImySQL'
        : 'DE_EPAGES::ShopTransfer::API::DBISybase';
    UsePackage( $DBIPackage );

    return bless {
        'dbh' => $DBIPackage->new,
    }, $class;
}

sub getDBIInstance {
    my $self = shift;
    return $self->{'dbh'};
}

#========================================================================================
# function     exportTable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exported = $dbi->exportTable($Table, $SiteID, $hOptions);
#----------------------------------------------------------------------------------------
# description  Exports all objects of a site from a database table to a file.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $SiteID | site id | integer
# input        $hOptions |
#                   <ul>
#                   <li>outPath - output directory - string </li>
#                   <li>encoding - character encoding - string </li>
#                   </ul> | ref.hash
# return       $Exported | true if the table was exported, false if the export
#                           file already exists | boolean
#========================================================================================
sub exportTable {
    my $self = shift;
    my ($Table, $SiteID, $hOptions) = @_;

    my $statement = $self->buildSelectForData($Table, $hOptions);

    my $FileName = $hOptions->{'outPath'}.'/'.$Table.'.bcp';
    DeleteFile($FileName) if ExistsFile($FileName) and $hOptions->{'replace'};
    return 0 if ExistsFile($FileName);

    my $dbi = GetCurrentDBHandle();
    my $dbh = $dbi->dbh;
    my $sth = $dbh->prepare($statement) or Error('DBIPrepare', {
            'message'   => $dbh->errstr,
            'statement' => $statement,
            'state'     => $dbh->state,
            'code'      => $dbh->err,
            'source'    => $dbi->source,
        });;

    if (index($statement,'?') > 0) {
        $sth->execute($SiteID) or Error('DBIExecute', {
            'message'   => $sth->errstr,
            'statement' => $statement,
            'parameter' => $SiteID,
            'state'     => $sth->state,
            'code'      => $sth->err,
            'source'    => $dbi->source,
        });
    } else {
        $sth->execute() or Error('DBIExecute', {
            'message'   => $sth->errstr,
            'statement' => $statement,
            'parameter' => $SiteID,
            'state'     => $sth->state,
            'code'      => $sth->err,
            'source'    => $dbi->source,
        });
    }
    BCPOutStatement($FileName, $sth, $hOptions->{'encoding'});
    $sth->finish;
    return 1;
}

sub buildSelectForData {
    my $self = shift;
    my ($Table) = @_;

    #
    # Build statement to select all necessary data for the $Table
    # and return it
    #
    my @PKeyColumns = $self->getDBIInstance()->getPKeyOfTable($Table);
    Error( 'TableWithoutPrimaryKey', {'Table' => $Table} ) if 0 == scalar @PKeyColumns;

    my @Columns = $self->getDBIInstance()->getColumns($Table);
    my $hTypes = $self->getDBIInstance()->getTypesOfColumns($Table);
    my $orderBy = ' ORDER BY ' . join(',', map { 't.'.$_} @PKeyColumns);

    my @GeneralizedDateColumns;
    foreach my $Column (@Columns) {
        if( $hTypes->{$Column} =~ /datetime/i ) {
            $Column = $self->getDBIInstance()->getGeneralizeDateTimeStatement($Column, 't.');
        }
        else {
            $Column = 't.' . $Column;
        }
        push @GeneralizedDateColumns, $Column;
    }

    my $ColumnString = join(',', @GeneralizedDateColumns);

    my $Column = $self->getDBIInstance->getObjectColumn( $Table );
    if ($Column eq '') {
        return sprintf 'SELECT %s FROM `%s` t %s', $ColumnString, $Table, $orderBy;
    }
    elsif( $Column eq 'shopid' or $Column eq 'siteid' ) {
        return sprintf 'SELECT %s FROM `%s` t WHERE t.%s = ? %s',
            $ColumnString, $Table, $Column,$orderBy;
    }
    else {
        my $dbi = GetCurrentDBHandle();
        my $exportids = $dbi->tempTableName('export_ids');
        return sprintf 'SELECT %s FROM `%s` t INNER JOIN %s o ON t.%s = o.objectid %s',
            $ColumnString, $Table, $exportids, $Column, $orderBy;
    }
}

1;
