﻿#========================================================================================
# §package      DE_EPAGES::Tax::API::TLE::TaxMatrixHandler
# §state        public
#----------------------------------------------------------------------------------------
# §description  Implements TLE functions for tax matrix
#========================================================================================
package DE_EPAGES::Tax::API::TLE::TaxMatrixHandler;
use base DE_EPAGES::TLE::API::LoopHandler;

use strict;
use DE_EPAGES::Core::API::Error qw (Error ExistsError GetError);
use DE_EPAGES::Core::API::Log qw(GetLog LogError);
use DE_EPAGES::Object::API::Factory qw(LoadObject);
use DE_EPAGES::Core::API::PerlTools qw (UsePackage);
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);
use utf8;

#========================================================================================
# §function     RegisterHandlerProc
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# §description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# §input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'TaxMatrixHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# §function     new
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Handler = new DE_EPAGES::Tax::API::TLE::TaxMatrixHandler;
# §example      my $PrefixHandler  = new DE_EPAGES::Tax::API::TLE::TaxMatrixHandler;
#----------------------------------------------------------------------------------------
# §description  create a new handler
#----------------------------------------------------------------------------------------
# §return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# §function     register
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# §description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# §input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'TaxMatrixHandler.register' );

    $Processor->registerHandler('FunctionHandler', $self, 'TaxAreaByCountryID');
    $Processor->registerHandler('FunctionHandler', $self, 'TaxRateForDefaultTaxArea');
    $Processor->registerHandler('FunctionHandler', $self, 'ZipcodesForCountry');
    $Processor->registerHandler('FunctionHandler', $self, 'IsMethodDirectlyUsable');
    $Processor->registerHandler('FunctionHandler', $self, 'DisplayTaxAreaAndRate');

    return;
}

#========================================================================================
# §function     TaxAreaByCountryID
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       #TaxAreaByCountryID(#TaxMatrix, #CountryID, #ZipcodeFrom, $ZipcodeTo)
# §example      #TaxAreaByCountryID(#Shop.TaxMatrix.Object, #CountryID, #ZipcodeFrom, $ZipcodeTo)
#----------------------------------------------------------------------------------------
# §description  Returns the TaxArea for a given country or region
#----------------------------------------------------------------------------------------
# §input        #TaxMatrix | taxmatrix object | object
# §input        #CountryID | country id | int
# §input        #ZipcodeFrom | min zipcode (optional) | string
# §input        #ZipcodeTo | max zipcode (optional) | string
#========================================================================================
sub TaxAreaByCountryID {
    my $self = shift;
    my ($Processor, $aParameter) = @_;
    my ($TaxMatrix, $CountryID, $ZipcodeFrom, $ZipcodeTo) = @$aParameter;
    Error( 'NoTaxMatrix' ) unless defined $TaxMatrix;
    Error( 'NoCountryID' ) unless defined $CountryID;
    return $TaxMatrix->taxAreaByCountry($CountryID)->tleHash if !defined $ZipcodeFrom || !defined $ZipcodeTo;
    return $TaxMatrix->taxAreaByCountry($CountryID, $ZipcodeFrom, $ZipcodeTo)->tleHash;
}

#========================================================================================
# §function     TaxRateForDefaultTaxArea
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       #TaxRateForDefaultTaxArea(#Shop, #TaxClass)
# §example      #TaxRateForDefaultTaxArea(#Shop.Object, #TaxClass.Object)
#----------------------------------------------------------------------------------------
# §description  Returns the tax rate for the tax class in default tax area of shop
#----------------------------------------------------------------------------------------
# §input        #Shop | shop | object
# §input        #TaxClass | tax class | object
#========================================================================================
sub TaxRateForDefaultTaxArea {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Shop, $TaxClass) = @$aParams;

    return unless $Shop;
    return unless $TaxClass;

    my $TaxMatrix = $TaxClass->get('TaxMatrix');

    my $Area = $TaxMatrix->defaultTaxArea();

    return $TaxMatrix->taxRate($Area, $TaxClass);
}

#========================================================================================
# §function     ZipcodesForCountry
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       #ZipcodesForCountry(#TaxMatrix, #CountryID)
# §example      #ZipcodesForCountry(#Shop.TaxMatrix.Object, #CountryID)
#----------------------------------------------------------------------------------------
# §description  Returns the Zipcodes with TaxArea for a country.
#----------------------------------------------------------------------------------------
# §input        #TaxMatrix | taxmatrix object | object
# §input        #CountryID | country id | int
#========================================================================================
sub ZipcodesForCountry {
    my $self = shift;
    my ($Processor, $aParameter) = @_;
    my ($TaxMatrix, $CountryID) = @$aParameter;
    return 'NoTaxMatrix' unless defined $TaxMatrix;
    return 'NoCountry' unless defined $CountryID;

    my @TaxAreaCountries;

    foreach my $hTaxAreaCountry (@{ $TaxMatrix->taxAreaCountries() }) {
        push @TaxAreaCountries, $hTaxAreaCountry if $hTaxAreaCountry->{'CountryID'} == $CountryID && defined $hTaxAreaCountry->{'ZipcodeFrom'};
    }
    foreach my $Entry (@TaxAreaCountries) {
        $Entry->{'TaxArea'} = LoadObject($Entry->{'TaxAreaID'})->tleHash;
    }
    return [sort { $a->{'ZipcodeFrom'} <=> $b->{'ZipcodeFrom'}} @TaxAreaCountries];
}

#========================================================================================
# §function     IsMethodDirectlyUsable
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       #IsMethodDirectlyUsable(#Shop, #TaxType)
# §example      #IsMethodDirectlyUsable(#Shop.Object, #TaxType.Object)
#----------------------------------------------------------------------------------------
# §description  Returns true if tax method by this tax type is usable without further configuration.
#----------------------------------------------------------------------------------------
# §input        #Shop | Shop object | object
# §input        #TaxType | TaxType id | int
#========================================================================================
sub IsMethodDirectlyUsable {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Shop, $TaxType) = @$aParams;

    # test if we have already a method object and use usableAtStorefront...
    foreach my $TaxMethod (@{$Shop->get('TaxMethods')}) {
        next if $TaxType->id != $TaxMethod->get('TaxType')->id;
        return $TaxMethod->usableAtStorefront() ? 1:0;
    }

    # we have not a method object, use function from TaxType package
    # load class package and call function without having an object
    my $Package = $TaxType->get('MethodClass')->get('Package');
    UsePackage($Package);

    return $Package->IsMethodDirectlyUsable($Shop);
}

#========================================================================================
# §function     DisplayTaxAreaAndRate
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       #DisplayTaxAreaAndRate(#Shop)
# §example      #DisplayTaxAreaAndRate(#Shop.Object)
#----------------------------------------------------------------------------------------
# §description  Returns true if taxarea and taxrate should be displayed.
#----------------------------------------------------------------------------------------
# §input        #Shop | Shop object | object
# §return       $displayed | true if should be displayed false otherwise | boolean
#========================================================================================
sub DisplayTaxAreaAndRate {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($BasketOrOrder) = @$aParams;

    my $result = eval {
        return $self->displayTaxAreaAndRate($BasketOrOrder);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        $result = 1;
    }

    return $result;
}

sub displayTaxAreaAndRate {
    my $self = shift;
    my ($BasketOrOrder) = @_;

    my $hInfo = {
        'BasketOrOrder' => $BasketOrOrder,
        'displayed' => 1,
    };

    $self->triggerHook('UI_DisplayTaxAreaAndRate', $hInfo);

    return $hInfo->{'displayed'};
}

sub triggerHook {
    my $self = shift;
    my ($HookName, $hParam) = @_;

    TriggerHook($HookName, $hParam);

    return;
}

1;
