#========================================================================================
# package      DE_EPAGES::WebService::API::JSONRestClient;
# state        public
# description  Generic client for RESTful web services using JSON messages.
#========================================================================================
package DE_EPAGES::WebService::API::JSONRestClient;

use strict;
use Data::Dumper;
use LWP::UserAgent;
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Core::API::TestParameter qw( TestArray TestString );
use DE_EPAGES::WebService::API::Codec::JSON;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Client = DE_EPAGES::WebService::API::JSONRestClient->new( $BaseURL, $hptions );
#----------------------------------------------------------------------------------------
# description  Creates a new REST client object.
#----------------------------------------------------------------------------------------
# input        $BaseURL | base URL | string
# input        $hOptions | <ul>
#                           <li>UserAgent - user agent object (default: LWP::UserAgent) - object</li>
#                           <li>Codec - content-type codec (default: DE_EPAGES::WebService::API::JSONCodec) - object</li>
#                           <li>BasicAuthentication - basic authentication parameters [ login, password ] - ref.array</li>
#                           </ul> | ref.hash
# return       $Client | client object | DE_EPAGES::WebService::API::JSONRestClient
#========================================================================================
sub new {
    my $class = shift;
    my ($BaseURL, $hOptions) = @_;
    TestString('BaseURL', $BaseURL);
    $hOptions //= {};
    TestArray('BasicAuthentication', $hOptions->{'BasicAuthentication'}) if defined $hOptions->{'BasicAuthentication'};

    return bless {
        BaseURL => $BaseURL,
        UserAgent => $hOptions->{'UserAgent'} // LWP::UserAgent->new,
        Codec => $hOptions->{'Codec'} // DE_EPAGES::WebService::API::Codec::JSON->new,
        BasicAuthentication => $hOptions->{'BasicAuthentication'},
    }, $class;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Codec = $Client->codec;
#----------------------------------------------------------------------------------------
# description  Returns the codec for decoding and encoding HTTP messages.
#----------------------------------------------------------------------------------------
# return       $Codec | codec object | DE_EPAGES::WebService::API::JSONCodec
#========================================================================================
sub codec {
    my $self = shift;

    return $self->{'Codec'};
}

#========================================================================================
# function     userAgent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $UserAgent = $Client->userAgent;
#----------------------------------------------------------------------------------------
# description  Returns the HTTP user agent.
#----------------------------------------------------------------------------------------
# return       $UserAgent | user agent object | LWP::UserAgent
#========================================================================================
sub userAgent {
    my $self = shift;

    return $self->{'UserAgent'};
}

#========================================================================================
# function     request
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ResponseData = $Client->request($Method, $SubPath, $RequestData);
#----------------------------------------------------------------------------------------
# description  Executes a REST request.
#----------------------------------------------------------------------------------------
# input        $Method | HTTP method | string
# input        $SubPath | String to append to the BaseURL | string
# input        $RequestData | request data | ref
# return       $ResponseData | decoded response data | ref
#========================================================================================
sub request {
    my $self = shift;
    my ($Method, $SubPath, $RequestData) = @_;

    my $Request = $self->buildRequest($Method, $SubPath, $RequestData);
    my $Response = $self->send($Request);
    return $self->decodeResponse($Response);
}

#========================================================================================
# function     send
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Response = $Client->send($Request);
#----------------------------------------------------------------------------------------
# description  Sends a HTTP request and return the corresponding HTTP response object.
#               See also <function buildRequest>.
#----------------------------------------------------------------------------------------
# input        $Request | request | HTTP::Request
# return       $Response | response | HTTP::Request
#========================================================================================
sub send {
    my $self = shift;
    my ($Request) = @_;

    return $self->userAgent->request( $Request );
}

#========================================================================================
# function     decodeResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ResponseData = $Client->decodeResponse($Response);
#----------------------------------------------------------------------------------------
# description  Extracts and decodes information from a HTTP response object.
#               Throws a HttpError exception if the response contains an error.
#               See also <function buildRequest>.
#----------------------------------------------------------------------------------------
# input        $Response | response object | HTTP::Response
# return       $ResponseData | decoded response data | ref
#========================================================================================
sub decodeResponse {
    my $self = shift;
    my ($Response) = @_;

    if( $Response->is_success ) {

        return undef unless $Response->content_length;

# does not work with content-type parameters, like ";version=..."
#        if( $Response->content_type ne $Codec->contentType ) {
#            Error('InvalidContentType', {
#                ContentType => scalar $Response->content_type,
#                Expected => $Codec->contentType,
#            });
#        }

        return $self->codec->decode($Response);
    }
    else {
        my $DecodedContent = eval { $self->codec->decode($Response) };
        Warning( GetError() ) if ExistsError();

        my $Dump = Data::Dumper->new([$DecodedContent], ['DecodedContent']);
        Error('HttpError', {
            Code => $Response->code,
            StatusLine => $Response->status_line,
            Response => $Response,
            DecodedContent => $Dump->Dump,
        });
    }
}

#========================================================================================
# function     buildRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Request = $Client->buildRequest($Method, $SubPath, $RequestData);
# example      my $Request = $Client->buildRequest($Method, $SubPath, $RequestData);
#               my $Response = $Client->send($Request);
#               my $Data = $Client->decodeResponse($Response);
#----------------------------------------------------------------------------------------
# description  Builds a HTTP request object for a REST call.
#----------------------------------------------------------------------------------------
# input        $Method | HTTP method | string
# input        $SubPath | String to append to the BaseURL | string
# input        $RequestData | request data | ref
# return       $Request | request object | HTTP::Request
#========================================================================================
sub buildRequest {
    my $self = shift;
    my ($Method, $SubPath, $RequestData) = @_;

    my $URL = $self->{BaseURL};
    $URL .= "/$SubPath" if defined $SubPath;

    my $Request = HTTP::Request->new($Method, $URL);

    my $Codec = $self->codec;
    $Request->header('Accept', $Codec->contentType);
    if ($self->{'BasicAuthentication'}) {
        $Request->authorization_basic(@{$self->{'BasicAuthentication'}});
    }
    $Codec->encode( $Request, $RequestData );

    return $Request;
}

#========================================================================================
# function     get
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Data = $Client->get($ID);
#----------------------------------------------------------------------------------------
# description  Executes a GET request.
#----------------------------------------------------------------------------------------
# input        $ID | resource id | string
# return       $Data | resource data | ref
#========================================================================================
sub get {
    my $self = shift;
    my ($ID) = @_;

    return $self->request('GET', $ID);
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Client->delete($ID);
#----------------------------------------------------------------------------------------
# description  Executes a DELETE request.
#----------------------------------------------------------------------------------------
# input        $ID | resource id | string
#========================================================================================
sub delete {
    my $self = shift;
    my ($ID) = @_;

    $self->request('DELETE', $ID);
    return;
}

#========================================================================================
# function     put
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Client->put($ID, $Data);
#----------------------------------------------------------------------------------------
# description  Executes a PUT request.
#----------------------------------------------------------------------------------------
# input        $ID | resource id | string
# input        $Data | resource data | ref
# input        $Return | 0, no return data, 1 otherwise | bool
#========================================================================================
sub put {
    my $self = shift;
    my ($ID, $Data, $Return) = @_;

    my $Result = $self->request('PUT', $ID, $Data);
    if($Return) {
        return $Result;
    }
    return;
}

#========================================================================================
# function     post
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ID = $Client->post($Data);
#----------------------------------------------------------------------------------------
# description  Executes a POST request.
#----------------------------------------------------------------------------------------
# input        $Data | resource data | ref
# return       $ID | resource id | string
#========================================================================================
sub post {
    my $self = shift;
    my ($Data) = @_;

    return $self->request('POST', undef, $Data);
}

#========================================================================================
# function     patch
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ID = $Client->patch($Data);
#----------------------------------------------------------------------------------------
# description  Executes a PATCH request.
#----------------------------------------------------------------------------------------
# input        $Data | resource data | ref
# return       $ID | resource id | string
#========================================================================================
sub patch {
    my $self = shift;
    my ($Data) = @_;

    return $self->request('PATCH', undef, $Data);
}

1;
