package DE_EPAGES::WebService::t::HelloResource;
use base DE_EPAGES::Database::API::Test;

use strict;
use Data::Dumper;
use Test::More tests => 44;
use HTTP::Status qw( HTTP_OK HTTP_NOT_ACCEPTABLE HTTP_METHOD_NOT_ALLOWED HTTP_NOT_FOUND );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::WebService::API::Codec::PlainText;
use DE_EPAGES::WebService::API::JSONRestClient;
use DE_EPAGES::WebService::Samples::Server::HelloResource;
use DE_EPAGES::WebService::API::ResourceRegistry;
use DE_EPAGES::WebInterface::API::FakeRequest;

use constant SITE_ALIAS => 'TestSite';
use constant DOMAIN_NAME => 'test-domain';

sub _TestHttpError(&$$) {
    my ( $Sub, $ExpectedStatus, $Message ) = @_;

    eval { $Sub->() };
    ok ExistsError(), "$Message: error expected";
    my $Error = GetError();
    is $Error->code, 'HttpError', "$Message: error code";
    is $Error->vars->{'Code'}, $ExpectedStatus, "$Message: HTTP status code=$ExpectedStatus"
}

sub init {
    my $self = shift;

    DE_EPAGES::WebService::API::ResourceRegistry->add(
        DE_EPAGES::WebService::Samples::Server::HelloResource->new
    );

    my $System = LoadRootObject();
    my $TestSite = $System->insertChild('Site', { Alias => SITE_ALIAS });
    $TestSite->set({
        DomainName => DOMAIN_NAME,
        WebServerScriptNamePart => SITE_ALIAS,
    });
    $self->{'TestSite'} = $TestSite;
}

sub cleanup {

    my $System = LoadRootObject();
    $System->child(SITE_ALIAS)->delete if $System->existsChild(SITE_ALIAS);
}

sub _resource {
    my $self = shift;
    my ($URL) = @_;

    return DE_EPAGES::WebService::API::JSONRestClient->new($URL, {
        UserAgent => DE_EPAGES::WebInterface::API::FakeRequest->new,
    });
}

sub _helloResource {
    my $self = shift;
    my ($DomainName) = @_;

    return $self->_resource("http://$DomainName/api/hello");
}

sub testUrl {
    my $self = shift;
    my ($URL, $Message) = @_;

    my $Resource = $self->_resource($URL);

    $Resource->put( 'OPetterson', {
        FirstName => 'Ole',
        LastName => 'Petterson',
    });

    my $hData = $Resource->get('OPetterson');
    is $hData->{'LastName'}, 'Petterson', "$Message: get";

    $Resource->delete('OPetterson');

    _TestHttpError sub { $Resource->get('OPetterson'); }, HTTP_NOT_FOUND, "$Message: delete";
}

# EPG-34475
sub testWithShortURL {
    my $self = shift;

    my $SystemDomain = LoadRootObject()->get('DomainName');
    $self->testUrl('http://'.$SystemDomain.'/epages/'.SITE_ALIAS.'.rest/api/hello', 'Shop with ShortURL');
}

# EPG-34475
sub testWithoutShortURL {
    my $self = shift;

    my $SystemDomain = LoadRootObject()->get('DomainName');
    my $StoreName = LoadRootObject()->get('StoreName');
    $self->testUrl("http://$SystemDomain/epages/$StoreName.rest/api/hello", 'Shop without ShortURL');
}

sub testWithUnknowDomain {
    my $self = shift;

    my $Resource = $self->_helloResource(DOMAIN_NAME);
    $Resource->put('UnknowDomain', { Name => 'Test' });
    is $Resource->get('UnknowDomain')->{'Name'}, 'Test', 'created a resource item';

    _TestHttpError sub { $self->_helloResource('dummy.org')->get('UnknowDomain'); }, HTTP_NOT_FOUND, 'unknown shop domain';

    $Resource->delete('UnknowDomain');
}

sub testUnknowResource {
    my $self = shift;

    my $Resource = $self->_resource('http://'.DOMAIN_NAME.'/api/dummy');

    _TestHttpError sub { $Resource->get('') }, HTTP_NOT_FOUND, 'unknown resource path';
}

sub testUnknowContentType {
    my $self = shift;

    my $HelloResource = DE_EPAGES::WebService::API::JSONRestClient->new('http://'.DOMAIN_NAME.'/api/hello', {
        UserAgent => DE_EPAGES::WebInterface::API::FakeRequest->new,
        Codec => DE_EPAGES::WebService::API::Codec::PlainText->new,
    });
    _TestHttpError sub { $HelloResource->get( 'TestID' ) }, HTTP_NOT_ACCEPTABLE, 'unknown content-type';
}

sub testUnknowRequestMethod {
    my $self = shift;

    my $Resource = $self->_helloResource(DOMAIN_NAME);
    _TestHttpError sub { $Resource->request('DUMMY', 'TestID') }, HTTP_METHOD_NOT_ALLOWED, 'DUMMY method';
}


sub testLowLevel {
    my $self = shift;

    my $Resource = $self->_helloResource(DOMAIN_NAME);

    $Resource->put( 'EOlsen', {
        FirstName => 'Egon',
        LastName => 'Olsen',
    });

    my $Request = $Resource->buildRequest('GET', 'EOlsen');
    is $Request->header('Accept'), 'application/json', 'buildRequest: accepted content-type';

    my $Response = $Resource->send($Request);
    is $Response->code, HTTP_OK, 'send: ok';

    my $hData = $Resource->decodeResponse($Response);
    is( $hData->{'LastName'}, 'Olsen', 'decodeResponse' );
}

sub testRequestMethods {
    my $self = shift;

    my $Resource = $self->_helloResource(DOMAIN_NAME);

    _TestHttpError sub { $Resource->get( 'TestID?locale=de_DE' ) }, HTTP_NOT_FOUND, 'empty collection';

    $Resource->put( 'TestID', {
        FirstName => 'Paul',
        LastName => 'Johnson',
    });

    my $hData = $Resource->get('TestID');
    is( $hData->{'FirstName'}, 'Paul', 'put+get' );

    $hData = $Resource->request('GET', 'TestID');
    is( $hData->{'FirstName'}, 'Paul', 'method(GET)' );

    $Resource->delete( 'TestID' );
    _TestHttpError sub { $Resource->get( 'TestID' ) }, HTTP_NOT_FOUND, 'delete';

    my $NewID = $Resource->post({
        FirstName => 'Paula',
        LastName => 'Steward',
    })->{'id'};
    isnt( $NewID, undef, 'post returns id' );
    $hData = $Resource->get( $NewID );
    is( $hData->{'LastName'}, 'Steward', 'post' );
}

sub testErrorResponse {
    my $self = shift;

    my $Resource = $self->_resource('http://'.DOMAIN_NAME.'/api/dummy');
    eval { $Resource->get(''); };

    ok ExistsError(), 'error response: error expected';
    my $Error = GetError();

    is $Error->code, 'HttpError', 'error response: error code';
    is $Error->vars->{'Code'}, HTTP_NOT_FOUND, 'error response: HTTP status code';
    like $Error->vars->{'StatusLine'}, qr/404 Not Found/, 'error response: StatusLine';
    is $Error->vars->{'Response'}->code, HTTP_NOT_FOUND, 'error response: Response';

    # read hash %$DecodedContent from string $Error->vars->{'DecodedContent'}
    my $DecodedContent;
    ## no critic qw(BuiltinFunctions::ProhibitStringyEval)
    eval $Error->vars->{'DecodedContent'};
    ## use critic

    is_deeply [sort keys %$DecodedContent], [qw(code data guid hash message)],
        'error response: DecodedContent' or Test::More::diag $Error->vars->{'DecodedContent'};
    my $H = '[0-9A-F]'; ## no critic qw(ProhibitUnusedVariablesEnhanced)
    my $h = '[0-9a-f]'; ## no critic qw(ProhibitUnusedVariablesEnhanced)
    like $DecodedContent->{guid}, qr/^$H{8}-$H{4}-$H{4}-$H{4}-$H{12}$/, 'error response: DecodedContent.guid';
    like $DecodedContent->{hash}, qr/^$h{32}$/, 'error response: DecodedContent.hash';
    is $DecodedContent->{code}, 'HandlerNotFound', 'error response: DecodedContent.code';
    is $DecodedContent->{message}, 'No resource handler found for "GET /api/dummy/"',
        'error response: DecodedContent.message';
}

sub test {
    my $self = shift;

    $self->testRequestMethods;
    $self->testLowLevel;
    $self->testWithUnknowDomain;
    $self->testWithShortURL; # for shops that don't have a domain name, but a short url
    $self->testWithoutShortURL; # for shops that have neither a domain name nor a short url
    $self->testUnknowResource;
    $self->testUnknowContentType;
    $self->testUnknowRequestMethod;
    $self->testErrorResponse;
}

__PACKAGE__->runScript;
